@@ -192,6 +192,10 @@ make_flag_bits(ModeInfo *mi)
   char *bitmap_name = get_string_resource (dpy, "bitmap", "Bitmap");
   char *text = get_string_resource (dpy, "text", "Text");
 
+#ifdef HAVE_COCOA
+  bitmap_name = 0;  /* #### always use default */
+#endif
+
   /* If neither a bitmap nor text are specified, randomly select either
 	 the builtin bitmap or builtin text. */
   if ((!bitmap_name || !*bitmap_name) && (!text || !*text))
@@ -403,6 +407,7 @@ init_flag(ModeInfo * mi)
 	fp = &flags[MI_SCREEN(mi)];
 
 	make_flag_bits(mi);
+    if (!fp->image) abort();
 
 	fp->width = MI_WIN_WIDTH(mi);
 	fp->height = MI_WIN_HEIGHT(mi);
@@ -472,6 +477,7 @@ draw_flag(ModeInfo * mi)
 	Window      window = MI_WINDOW(mi);
 	flagstruct *fp = &flags[MI_SCREEN(mi)];
 
+    if (!fp->image) abort();
     if (fp->cache == window) {  /* not double-buffering */
       XClearWindow (display, window);
     } else if (fp->width <= MAXW(fp) || fp->height <= MAXH(fp)) {
