@@ -55,6 +55,13 @@
  */
 struct lockward_context;			/*  Forward declaration.  */
 
+#define int8_t   char
+#define int16_t  short
+#define int32_t  int
+#define uint8_t  unsigned char
+#define uint16_t unsigned short
+#define uint32_t unsigned int
+
 typedef struct bladestate {
 	uint8_t		outer, inner;	/*  Radii  */
 } bladestate;
