@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 1992-2006 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 1992-2008 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -581,7 +581,7 @@ run_screenhack_table (Display *dpy,
     {
       unsigned long delay = ft->draw_cb (dpy, window, closure);
 #ifdef DEBUG_PAIR
-      unsigned long delay2;
+      unsigned long delay2 = 0;
       if (window2) delay2 = ft->draw_cb (dpy, window2, closure2);
 #endif
 
@@ -684,7 +684,7 @@ init_window (Display *dpy, Widget toplevel, const char *title)
   window = XtWindow (toplevel);
   XGetWindowAttributes (dpy, window, &xgwa);
   XSelectInput (dpy, window,
-                (xgwa.your_event_mask | KeyPressMask |
+                (xgwa.your_event_mask | KeyPressMask | KeyReleaseMask |
                  ButtonPressMask | ButtonReleaseMask));
   XChangeProperty (dpy, window, XA_WM_PROTOCOLS, XA_ATOM, 32,
                    PropModeReplace,
