@@ -1284,14 +1284,14 @@ hack_text (m_state *state)
 # endif
               ("# "
                "\010\010\010\010"
-               "\001nmap 10.2.2.2\n"
+               "\001nmap -v -sS -O 10.2.2.2\n"
                "Starting nmap V. 2.54BETA25\n"
                "\010\010\010\010\010\010\010\010\010\010"
                "Insufficient responses for TCP sequencing (3), OS detection"
                " may be less accurate\n"
                "Interesting ports on 10.2.2.2:\n"
-               "(The 1538 ports scanned but not shown below are in state:"
-               " filtered)\n"
+               "(The 1539 ports scanned but not shown below are in state:"
+               " closed)\n"
                "Port       state       service\n"
                "22/tcp     open        ssh\n"
                "\n"
